#Replication file for:
#John A. Hamman, Shane A. Gleason, and Charles DiStefano:
#Incumbent vulnerability and entry in gubernatorial elections: The case of former governors
#In Social Science Journal

#Script by: Shane A. Gleason
#January 22, 2014

#This analysis ran in R.  For the additional tables and models ran as controls 
#in the paper please contact Shane Gleason

#Load the required packages
require(lme4)  #to run the actual models
require(epicalc) # to calculate the area under the ROC curve

#Read in the data
govs <- read.csv("incumbents.csv")

##################################################
#Reproduce Table 1:
final.model <- glm(comeback ~ quality_cand + incumbent + 
                     unemployment + spend_pop + quality_cand + laglog + 
                     unemployment +  pool + I(laglog*incumbent) + I(unemployment*incumbent)  
                   + I(incumbent*quality_cand), family=binomial("logit"), data=govs)
summary(final.model)

logLik(final.model)
#get the ROC
par(mfrow=c(1,1)) 
lroc(final.model, title=TRUE, auc.coords=c(.5,.1)) #Grab percentage from the graphic for ROC
exp(coef(final.model)) #raw odds ratios


###################################################
#Reproduce Figure 1:
par(mfrow=c(1,2)) 
hist(govs$laglog, xlab="", main="Logged Vote Lag")
hist(govs$lagfix, xlab="", main="Square Root of Vote Lag")


######################################################
######################################################
#Graphing the interaction terms:
#First, we estimate models for each (the order of the variables matters for graphing purposes- results no 
#different than final.model reported above)

#Unemployment:
logit.unemploy <- glm(comeback ~ unemployment + incumbent + quality_cand + 
                       spend_pop + laglog + 
                       pool + I(laglog*incumbent) 
                     + I(incumbent*quality_cand) + I(unemployment*incumbent)  , family=binomial("logit"), data=govs)
#Vote spread (logged)
logit.laglog <- glm(comeback ~ laglog + incumbent + 
                     spend_pop + quality_cand + unemployment +
                     pool  
                   + I(incumbent*quality_cand) + I(unemployment*incumbent)  + I(laglog*incumbent)  , family=binomial("logit"), data=govs)

#Quality candidate pool
logit.quality <- glm(comeback ~ quality_cand + incumbent + unemployment + 
                       spend_pop + laglog + 
                       pool  + 
                       I(laglog*incumbent) +  I(unemployment*incumbent) + I(incumbent*quality_cand)  , family=binomial("logit"), data=govs)

###################################################
#Render Figure 2--- Lagged Vote Spread
par(mfrow=c(1,1)) 

##############################################
## Variables must be in the following order: y = x z (control variables here) xz .  The model can include as many control variables as you need.
## M = an object of type "lm," "glm," or other estimation -- i.e. the object that contains the regression estimation you seek to plot
## X = the variable whose effect on Y you seek to plot
## Z = the moderating variable (will be positioned on the X-axis of the plot)
## xlab = label for x-axis (in quotes)
## ylab = label for y-axis (in quotes)
## level = to set the confidence level.  Two options (don't put these in quotes): 95, 90.  If you do not specify either option, the confidence intervals will not be correct.
xlab <- "Log of Vote Spread"
ylab <- "Marginal Effect of Incumbency"
main <- "Interaction of Lagged Vote Spread and Incumbency"
variable <- govs$unemployment
## Example: TwowayME.f(estimation.lm,ses,edu,"Education levels","Effect of SES on Civil War",90)

S <- summary(logit.laglog)
N <- c(1:20)

#     ################################################################  #
#       Create 20 equally spaced values in a vector between min         #
#       and max on the Z variable                                       #
#     ################################################################  #

zmin <- rep(min(variable,na.rm=TRUE), 20)
zmax <- rep(max(variable, na.rm=TRUE), 20)
Z <- (((N-1)/(20-1))*(zmax-zmin))+zmin

#     ################################################################  #
#       Grab elements of the coefficient and variance-covariance matrix #
#       that are required to calculate the marginal effect and standard #
#       errors.                                                         #
#     ################################################################  #

H <- head(S$coefficients,3)
T <- tail(S$coefficients,1)
b <- rbind(H,T)
Vcov <- vcov(logit.laglog)
Vcov <- as.data.frame(Vcov)
Vcov1 <- Vcov[,c(1:3)]
Vcov2 <- Vcov[,-c(0:0-length(Vcov))]
Vcov <- cbind(Vcov1,Vcov2)
Vh <- head(Vcov,3)
Vt <- tail(Vcov,1)
V <- rbind(Vh,Vt)

b1 <- b[2,1]
b2 <- b[3,1]
b3 <- b[4,1]

varb1 <- V[2,2]
varb2 <- V[3,3]
varb3 <- V[4,4]

covb1b3 <- V[4,2]
covb2b3 <- V[4,3]

#     ################################################################  #
#       We want to calculate the marginal effect of X on Y for all      #
#       values of the modifying variable Z.                             #
#     ################################################################  #

conb <- b1+b3*Z

#     ################################################################  #
#       Calculate the standard errors for the marginal effect of X on Y #
#       for all Z values of the modifying variable Z. Do this for the   #
#       case when W=0, when W=1, when W=2, when W=3, and when W=4.      #
#     ################################################################  #

conse <- sqrt(varb1 + varb3*(Z^2) + 2*covb1b3*Z)

#     ################################################################  #
#              Generate Upper and Lower Confidence Intervals            #
#                       -- specify significance                         #
#     ################################################################  #

ci <- 1.96
#ci[level==95] <- 1.96
#ci[level==90] <- 1.645

a = ci*conse
upper = conb + a
lower = conb - a

#     ################################################################  #
#       Graph the marginal effect of X on Y across the desired range of #
#       the modifying variable Z.                                       #
#     ################################################################  #
plot(c(Z,Z,Z), c(a,upper,lower), type="n",xlab=xlab,ylab=ylab, main=main)
lines(Z,conb,col="black")
lines(Z,upper,col="black",lty=2)
lines(Z,lower,col="black",lty=2)
abline(h=0)

box()
mtext("95% Confidence Interval")






###################################################
#Render Figure 3---- Unemployment
par(mfrow=c(1,2)) 
##############################################
## Variables must be in the following order: y = x z (control variables here) xz .  The model can include as many control variables as you need.
## M = an object of type "lm," "glm," or other estimation -- i.e. the object that contains the regression estimation you seek to plot
## X = the variable whose effect on Y you seek to plot
## Z = the moderating variable (will be positioned on the X-axis of the plot)
## xlab = label for x-axis (in quotes)
## ylab = label for y-axis (in quotes)
## level = to set the confidence level.  Two options (don't put these in quotes): 95, 90.  If you do not specify either option, the confidence intervals will not be correct.
xlab <- "Change in Unemployment"
ylab <- "Marginal Effect of Incumbency"
main <- "Interaction of Change in Unemployment and Incumbency"
variable <- govs$unemployment
## Example: TwowayME.f(estimation.lm,ses,edu,"Education levels","Effect of SES on Civil War",90)

S <- summary(logit.unemploy)
N <- c(1:20)

#     ################################################################  #
#       Create 20 equally spaced values in a vector between min         #
#       and max on the Z variable                                       #
#     ################################################################  #

zmin <- rep(min(variable,na.rm=TRUE), 20)
zmax <- rep(max(variable, na.rm=TRUE), 20)
Z <- (((N-1)/(20-1))*(zmax-zmin))+zmin

#     ################################################################  #
#       Grab elements of the coefficient and variance-covariance matrix #
#       that are required to calculate the marginal effect and standard #
#       errors.                                                         #
#     ################################################################  #

H <- head(S$coefficients,3)
T <- tail(S$coefficients,1)
b <- rbind(H,T)
Vcov <- vcov(logit.unemploy)
Vcov <- as.data.frame(Vcov)
Vcov1 <- Vcov[,c(1:3)]
Vcov2 <- Vcov[,-c(0:0-length(Vcov))]
Vcov <- cbind(Vcov1,Vcov2)
Vh <- head(Vcov,3)
Vt <- tail(Vcov,1)
V <- rbind(Vh,Vt)

b1 <- b[2,1]
b2 <- b[3,1]
b3 <- b[4,1]

varb1 <- V[2,2]
varb2 <- V[3,3]
varb3 <- V[4,4]

covb1b3 <- V[4,2]
covb2b3 <- V[4,3]

#     ################################################################  #
#       We want to calculate the marginal effect of X on Y for all      #
#       values of the modifying variable Z.                             #
#     ################################################################  #

conb <- b1+b3*Z

#     ################################################################  #
#       Calculate the standard errors for the marginal effect of X on Y #
#       for all Z values of the modifying variable Z. Do this for the   #
#       case when W=0, when W=1, when W=2, when W=3, and when W=4.      #
#     ################################################################  #

conse <- sqrt(varb1 + varb3*(Z^2) + 2*covb1b3*Z)

#     ################################################################  #
#              Generate Upper and Lower Confidence Intervals            #
#                       -- specify significance                         #
#     ################################################################  #

ci <- 1.645
#ci[level==95] <- 1.96
#ci[level==90] <- 1.645

a = ci*conse
upper = conb + a
lower = conb - a

#     ################################################################  #
#       Graph the marginal effect of X on Y across the desired range of #
#       the modifying variable Z.                                       #
#     ################################################################  #
plot(c(Z,Z,Z), c(a,upper,lower), type="n",xlab=xlab,ylab=ylab, main=main)
lines(Z,conb,col="black")
lines(Z,upper,col="black",lty=2)
lines(Z,lower,col="black",lty=2)
abline(h=0)

box()
mtext("90% Confidence Interval")

#Now, redo it with a 95pct CI

##############################################
## Variables must be in the following order: y = x z (control variables here) xz .  The model can include as many control variables as you need.
## M = an object of type "lm," "glm," or other estimation -- i.e. the object that contains the regression estimation you seek to plot
## X = the variable whose effect on Y you seek to plot
## Z = the moderating variable (will be positioned on the X-axis of the plot)
## xlab = label for x-axis (in quotes)
## ylab = label for y-axis (in quotes)
## level = to set the confidence level.  Two options (don't put these in quotes): 95, 90.  If you do not specify either option, the confidence intervals will not be correct.
xlab <- "Change in Unemployment"
ylab <- "Marginal Effect of Incumbency"
main <- "Interaction of Change in Unemployment and Incumbency"
variable <- govs$unemployment
## Example: TwowayME.f(estimation.lm,ses,edu,"Education levels","Effect of SES on Civil War",90)

S <- summary(logit.unemploy)
N <- c(1:20)

#     ################################################################  #
#       Create 20 equally spaced values in a vector between min         #
#       and max on the Z variable                                       #
#     ################################################################  #

zmin <- rep(min(variable,na.rm=TRUE), 20)
zmax <- rep(max(variable, na.rm=TRUE), 20)
Z <- (((N-1)/(20-1))*(zmax-zmin))+zmin

#     ################################################################  #
#       Grab elements of the coefficient and variance-covariance matrix #
#       that are required to calculate the marginal effect and standard #
#       errors.                                                         #
#     ################################################################  #

H <- head(S$coefficients,3)
T <- tail(S$coefficients,1)
b <- rbind(H,T)
Vcov <- vcov(logit.unemploy)
Vcov <- as.data.frame(Vcov)
Vcov1 <- Vcov[,c(1:3)]
Vcov2 <- Vcov[,-c(0:0-length(Vcov))]
Vcov <- cbind(Vcov1,Vcov2)
Vh <- head(Vcov,3)
Vt <- tail(Vcov,1)
V <- rbind(Vh,Vt)

b1 <- b[2,1]
b2 <- b[3,1]
b3 <- b[4,1]

varb1 <- V[2,2]
varb2 <- V[3,3]
varb3 <- V[4,4]

covb1b3 <- V[4,2]
covb2b3 <- V[4,3]

#     ################################################################  #
#       We want to calculate the marginal effect of X on Y for all      #
#       values of the modifying variable Z.                             #
#     ################################################################  #

conb <- b1+b3*Z

#     ################################################################  #
#       Calculate the standard errors for the marginal effect of X on Y #
#       for all Z values of the modifying variable Z. Do this for the   #
#       case when W=0, when W=1, when W=2, when W=3, and when W=4.      #
#     ################################################################  #

conse <- sqrt(varb1 + varb3*(Z^2) + 2*covb1b3*Z)

#     ################################################################  #
#              Generate Upper and Lower Confidence Intervals            #
#                       -- specify significance                         #
#     ################################################################  #

ci <- 1.96
#ci[level==95] <- 1.96
#ci[level==90] <- 1.645

a = ci*conse
upper = conb + a
lower = conb - a

#     ################################################################  #
#       Graph the marginal effect of X on Y across the desired range of #
#       the modifying variable Z.                                       #
#     ################################################################  #
plot(c(Z,Z,Z), c(a,upper,lower), type="n",xlab=xlab,ylab=ylab, main=main)
lines(Z,conb,col="black")
lines(Z,upper,col="black",lty=2)
lines(Z,lower,col="black",lty=2)
abline(h=0)

box()
mtext("95% Confidence Interval")

####################################################################
#Render Figure 4---- Quality candidates

par(mfrow=c(1,1)) 


##############################################
## Variables must be in the following order: y = x z (control variables here) xz .  The model can include as many control variables as you need.
## M = an object of type "lm," "glm," or other estimation -- i.e. the object that contains the regression estimation you seek to plot
## X = the variable whose effect on Y you seek to plot
## Z = the moderating variable (will be positioned on the X-axis of the plot)
## xlab = label for x-axis (in quotes)
## ylab = label for y-axis (in quotes)
## level = to set the confidence level.  Two options (don't put these in quotes): 95, 90.  If you do not specify either option, the confidence intervals will not be correct.
xlab <- "Number of Quality Candidates in Primary"
ylab <- "Marginal Effect of Incumbency"
main <- "Interaction of Number of Quality Candidates and Incumbency"
variable <- govs$unemployment
## Example: TwowayME.f(estimation.lm,ses,edu,"Education levels","Effect of SES on Civil War",90)

S <- summary(logit.quality)
N <- c(1:20)

#     ################################################################  #
#       Create 20 equally spaced values in a vector between min         #
#       and max on the Z variable                                       #
#     ################################################################  #

zmin <- rep(min(variable,na.rm=TRUE), 20)
zmax <- rep(max(variable, na.rm=TRUE), 20)
Z <- (((N-1)/(20-1))*(zmax-zmin))+zmin

#     ################################################################  #
#       Grab elements of the coefficient and variance-covariance matrix #
#       that are required to calculate the marginal effect and standard #
#       errors.                                                         #
#     ################################################################  #

H <- head(S$coefficients,3)
T <- tail(S$coefficients,1)
b <- rbind(H,T)
Vcov <- vcov(logit.quality)
Vcov <- as.data.frame(Vcov)
Vcov1 <- Vcov[,c(1:3)]
Vcov2 <- Vcov[,-c(0:0-length(Vcov))]
Vcov <- cbind(Vcov1,Vcov2)
Vh <- head(Vcov,3)
Vt <- tail(Vcov,1)
V <- rbind(Vh,Vt)

b1 <- b[2,1]
b2 <- b[3,1]
b3 <- b[4,1]

varb1 <- V[2,2]
varb2 <- V[3,3]
varb3 <- V[4,4]

covb1b3 <- V[4,2]
covb2b3 <- V[4,3]

#     ################################################################  #
#       We want to calculate the marginal effect of X on Y for all      #
#       values of the modifying variable Z.                             #
#     ################################################################  #

conb <- b1+b3*Z

#     ################################################################  #
#       Calculate the standard errors for the marginal effect of X on Y #
#       for all Z values of the modifying variable Z. Do this for the   #
#       case when W=0, when W=1, when W=2, when W=3, and when W=4.      #
#     ################################################################  #

conse <- sqrt(varb1 + varb3*(Z^2) + 2*covb1b3*Z)

#     ################################################################  #
#              Generate Upper and Lower Confidence Intervals            #
#                       -- specify significance                         #
#     ################################################################  #

ci <- 1.96
#ci[level==95] <- 1.96
#ci[level==90] <- 1.645

a = ci*conse
upper = conb + a
lower = conb - a

#     ################################################################  #
#       Graph the marginal effect of X on Y across the desired range of #
#       the modifying variable Z.                                       #
#     ################################################################  #
plot(c(Z,Z,Z), c(a,upper,lower), type="n",xlab=xlab,ylab=ylab, main=main,xlim=c(0,3), ylim=c(-5,10))
lines(Z,conb,col="black")
lines(Z,upper,col="black",lty=2)
lines(Z,lower,col="black",lty=2)
abline(h=0)

box()
mtext("95% Confidence Interval")

